kBRANCH_WORKING := working
kBRANCH_DEVELOP := develop
kBRANCH_TESTING := testing
kBRANCH_MASTER := master

#
# define REPO_BRANCH, REPO_MAJOR, REPO_MINOR, REPO_PATCH
BRANCH_OR_HEAD := $(shell git rev-parse --abbrev-ref HEAD)
ifeq "$(BRANCH_OR_HEAD)" "HEAD"
    # Case of git-tag <stable-x.y.z>.
    LAST_TAG := $(shell git describe --tags --always)
    ifeq "$(filter stable-%, $(LAST_TAG))" ""
        $(error invalid tag <$(LAST_TAG)>)
    endif

    MAJOR_MINOR_PATCH := $(subst ., ,$(patsubst stable-%,%,$(LAST_TAG)))
    REPO_MAJOR ?= $(word 1, $(MAJOR_MINOR_PATCH))
    REPO_MINOR ?= $(word 2, $(MAJOR_MINOR_PATCH))
    REPO_PATCH ?= $(word 3, $(MAJOR_MINOR_PATCH))
    REPO_BRANCH := $(kBRANCH_MASTER)

else ifneq "$(filter testing-%, $(BRANCH_OR_HEAD))" ""
    # Case of git-branch <testing-x.y.z>.
    MAJOR_MINOR_PATCH := $(subst ., ,$(patsubst testing-%,%,$(BRANCH_OR_HEAD)))
    REPO_MAJOR ?= $(word 1, $(MAJOR_MINOR_PATCH))
    REPO_MINOR ?= $(word 2, $(MAJOR_MINOR_PATCH))
    REPO_PATCH ?= $(word 3, $(MAJOR_MINOR_PATCH))
    REPO_BRANCH := $(kBRANCH_TESTING)

else ifeq "$(kBRANCH_DEVELOP)" "$(BRANCH_OR_HEAD)"
    # Case of git-branch <develop>.
    LAST_TAG := $(shell git describe --tags $(shell git rev-list --tags --max-count=1))
    MAJOR_MINOR_PATCH := $(subst ., ,$(patsubst stable-%,%,$(LAST_TAG)))
    REPO_MAJOR ?= $(word 1, $(MAJOR_MINOR_PATCH))
    REPO_MINOR ?= $(shell expr $(word 2, $(MAJOR_MINOR_PATCH)) + 1)
    REPO_PATCH ?= 0
    REPO_BRANCH := $(kBRANCH_DEVELOP)

else ifeq "$(kBRANCH_MASTER)" "$(BRANCH_OR_HEAD)"
    # Case of git-branch <master>.
    LAST_TAG := $(shell git describe --tags $(shell git rev-list --tags --max-count=1))
    MAJOR_MINOR_PATCH := $(subst ., ,$(patsubst stable-%,%,$(LAST_TAG)))
    REPO_MAJOR ?= $(word 1, $(MAJOR_MINOR_PATCH))
    REPO_MINOR ?= $(word 2, $(MAJOR_MINOR_PATCH))
    REPO_PATCH ?= $(word 3, $(MAJOR_MINOR_PATCH))
    REPO_BRANCH := $(kBRANCH_MASTER)

else
    # Case of personal git-branch
    MAJOR_MINOR_PATCH := $(subst ., ,$(patsubst experimental-%,%,experimental-0.0.0))
    REPO_MAJOR ?= $(word 1, $(MAJOR_MINOR_PATCH))
    REPO_MINOR ?= $(word 2, $(MAJOR_MINOR_PATCH))
    REPO_PATCH ?= $(word 3, $(MAJOR_MINOR_PATCH))
    REPO_BRANCH := $(BRANCH_OR_HEAD)

endif

#
# define VER_DIST, VER_MAJOR, VER_MINOR, VER_PATCH, VER_BUILD
VER_DIST ?= experimental
VER_MAJOR ?= $(REPO_MAJOR)
VER_MINOR ?= $(REPO_MINOR)
VER_PATCH ?= $(REPO_PATCH)
VER_BUILD ?= 0

ifneq ($(shell test $(VER_MAJOR) -ge 0; echo $$?),0)
    $(error invalid ver-major <$(VER_MAJOR)>)
else ifneq ($(shell test $(VER_MINOR) -ge 0; echo $$?),0)
    $(error invalid ver-minor <$(VER_MINOR)>)
else ifneq ($(shell test $(VER_PATCH) -ge 0; echo $$?),0)
    $(error invalid ver-patch <$(VER_PATCH)>)
endif


#
# define SEM_VER, DCH_VER, DCH_REV and validate REPO_BRANCH
DCH_REV := $(shell git log --pretty=format:'%H' -n 1)
ifeq "$(VER_DIST)" "experimental"
    SEM_VER := $(VER_MAJOR).$(VER_MINOR).$(VER_PATCH)
    DCH_VER := $(VER_MAJOR).$(VER_MINOR).$(VER_PATCH)-1~~~$(VER_DIST)+$(VER_BUILD)

else ifeq "$(VER_DIST)" "unstable"
    SEM_VER := $(VER_MAJOR).$(VER_MINOR).0
    DCH_VER := $(VER_MAJOR).$(VER_MINOR).0-1~~$(VER_DIST)+$(VER_BUILD)
    ifneq "$(REPO_BRANCH)" "$(kBRANCH_DEVELOP)"
        $(error Can not make $(VER_DIST) distr from $(REPO_BRANCH) branch)
    endif

else ifeq "$(VER_DIST)" "testing"
    SEM_VER := $(VER_MAJOR).$(VER_MINOR).$(VER_PATCH)
    DCH_VER := $(VER_MAJOR).$(VER_MINOR).$(VER_PATCH)-1~$(VER_DIST)+$(VER_BUILD)
    ifneq "$(REPO_BRANCH)" "$(kBRANCH_TESTING)"
        $(error Can not make $(VER_DIST) distr from $(REPO_BRANCH) branch)
    endif

else ifeq "$(VER_DIST)" "stable"
    SEM_VER := $(VER_MAJOR).$(VER_MINOR).$(VER_PATCH)
    DCH_VER := $(VER_MAJOR).$(VER_MINOR).$(VER_PATCH)-1+$(VER_BUILD)
    ifneq "$(REPO_BRANCH)" "$(kBRANCH_MASTER)"
        $(error Can not make $(VER_DIST) distr from $(REPO_BRANCH) branch)
    endif

else
    $(error invalid dist <$(VER_DIST)>)

endif


.PHONY: whoami
whoami:
	@echo "\n\
	dist:          $(VER_DIST)\n\
	repo-branch:   $(REPO_BRANCH)\n\
	repo-sem-ver:  $(REPO_MAJOR).$(REPO_MINOR).$(REPO_PATCH)\n\
	next-sem-ver:  $(SEM_VER)\n\
	next-dch-ver:  $(DCH_VER)\n"
