#!/bin/bash
# This script is used to lock/unlock edge-db PostgreSQL cluster during install/update
_usage() {
    echo "Usage: $0 -m lock|unlock [-d dbname] [-u username]"
    echo " NB! For default values see config/*.config!"
    echo "   -C, --config         Configuration name"
    echo "   -m, --mode           Mainteanance mode, available modes: lock|unlock"
    echo "   -d, --database       PostgreSQL database name"
    echo "   -u, --user           PostgreSQL user name"
    echo "   -?, --help           This help text"
    exit 0
}

# Must be root to run this script
if test "$EUID" -ne 0 ; then
    echo "Error: Please run as root!"
    exit -1
fi

# Current dir
_ME=$(readlink -f "$0")
PWD=$(dirname "${_ME}")
LOCK=$PWD/maintenance.lock

# Config name
[ -f ${PWD}/.configure ] && CONFIG=$(cat ${PWD}/.configure | tr -d "\n\r")
CONFIG=${CONFIG:-default}

# Selected mode
MMODE=

# CLI params
while test $# -gt 0; do
    case $1 in
        --config|-C)
            CONFIG="${2}"
            shift
        ;;
        --mode|-m)
            MMODE="${2}"
            shift
        ;;
        --database|-d)
            PGDBNAME="${2}"
            shift
        ;;
        --user|-u)
            PGUSER="${2}"
            shift
        ;;
        --help|-?)
            _usage
            break
        ;;
        --)
            break
        ;;
        --*|-?)
            echo "Unknown option ${1}"
            exit 11
        ;;
        -*)
            split=$1
            shift
            set -- $(echo "$split" | cut -c 2- | sed 's/./-& /g') "$@"
            continue
        ;;
        *)
            echo "Unknown argument ${1}"
            exit 11
        ;;
    esac

    shift
done

# Configs
source $PWD/config/$CONFIG.config

# Main subroutines
_maintenance_lock() {
    echo "0" > $LOCK
    while [ 1 ]  
    do
        ps -ax | grep psql | grep -q -v grep || break
        pkill -f psql
        sleep 2
    done
    psql -q -t -c "select pg_terminate_backend(pg_stat_activity.pid) from pg_stat_activity where datname = current_database() and pid <> pg_backend_pid();" ${PGDBNAME} ${PGUSER} > ${PGLOGDIR}/${PGDBNAME}.maintenance.lock.log 2>&1
}

_maintenance_unlock() {
    rm -f $LOCK
}


if [ "${MMODE}" == "lock" ]; then _maintenance_lock; exit 0; fi
if [ "${MMODE}" == "unlock" ]; then _maintenance_unlock; exit 0; fi

# If unknown mode is provided or no mode at all - echo usage
_usage
