
create or replace function binfs._statdir ( idir text, idepth integer ) returns json as $$
import json

def _statappend(ito, ifrom):
    ito['ocount'] += ifrom['ocount']
    ito['osize'] += ifrom['osize']
    if not ito['oftimemin'] or ifrom['oftimemin'] < ito['oftimemin']:
        ito['oftimemin'] = ifrom['oftimemin']
    if not ito['oftimemax'] or ifrom['oftimemax'] > ito['oftimemax']:
        ito['oftimemax'] = ifrom['oftimemax']
    if not ito['opathmin'] or ifrom['opathmin'] < ito['opathmin']:
        ito['opathmin'] = ifrom['opathmin']
    if not ito['opathmax'] or ifrom['opathmax'] > ito['opathmax']:
        ito['opathmax'] = ifrom['opathmax']

def _dirproc(ipath, idepth):
    import os

    stsum = {
        "ocount": 0,
        "osize": 0,
        "oftimemin": None,
        "oftimemax": None,
        "opathmin": None,
        "opathmax": None
    }
    stcurrent = {
        "ocount": 0,
        "osize": 0,
        "oftimemin": None,
        "oftimemax": None,
        "opathmin": None,
        "opathmax": None
    }
    if not os.path.exists(ipath):
        return stsum
    rescan = True
    if idepth > 0:
        for f in os.listdir(ipath):
            if f.startswith('.'):
                continue
            if f.startswith('__binfs'):
                continue
            if os.path.isdir(os.path.join(ipath, f)):
                stats = _dirproc(os.path.join(ipath, f), idepth - 1)
                _statappend(stsum, stats)
    else:
        pathdir, pathname = os.path.split(ipath)
        st = os.lstat(ipath)
        sto = dict((name, getattr(st, name)) for name in dir(st) if
                   not name.startswith('__'))
        sth = "%x" % (hash(tuple(sorted(hash(x) for x in sto.items()))) &
                      0xffffffffffffffff)
        stfile = os.path.join(pathdir, '.stat.{0}.{1}'.format(pathname, sth))
        if os.path.exists(stfile):
            try:
                with open(stfile, 'r') as stfd:
                    stf = json.load(stfd)
                rescan = False
                stcurrent = stf['stats']
            except Exception as ex:
                plpy.execute("select core.log('binfs._statdir',"
                             "'error','prescan: '||{0})"
                             .format(plpy.quote_literal(str(ex))))
                os.remove(stfile)

        if rescan:
            try:
                for f in os.listdir(pathdir):
                    if f.startswith('.stat.' + pathname):
                        os.unlink(os.path.join(pathdir, f))
                for f in os.listdir(ipath):
                    if f.startswith('.'):
                        continue
                    if f.startswith('__binfs'):
                        continue
                    sst = os.lstat(os.path.join(ipath, f))
                    cstats = {
                        "ocount": 1,
                        "osize": sst.st_size,
                        "oftimemin": sst.st_ctime,
                        "oftimemax": sst.st_ctime,
                        "opathmin": f,
                        "opathmax": f
                    }
                    _statappend(stcurrent, cstats)
                with open(stfile, 'w') as stfd:
                    json.dump({"stat": sto, "stats": stcurrent}, stfd,
                              sort_keys=True, indent=4, ensure_ascii=False)
            except Exception as ex:
                plpy.execute("select core.log('binfs._statdir',"
                             "'error','rescan: '||{0})"
                             .format(plpy.quote_literal(str(ex))))
                plpy.error(plpy.quote_literal(str(ex)))
        _statappend(stsum, stcurrent)
    return stsum

try:
    res = json.dumps(_dirproc(idir, idepth))
except Exception as ex:
    plpy.execute("select core.log('binfs._statdir',"
                 "'error','json: '||{0})".format(plpy.quote_literal(str(ex))))
    return None
return res
$$ language plpythonu;

--
--
--

create or replace function binfs._stat ( 
	istore text
	,out "size" bigint
	,out count bigint
	,out ftimemin timestamptz
	,out ftimemax timestamptz
	,out pathmin text
	,out pathmax text
) as $$

import json

from datetime import datetime

try:
    param = json.loads(istore)
    root = param['root']
    kind = param['class']
    opt = eval(param['opt'])
except Exception as ex:
    plpy.execute("select core.log('binfs._stat','error','opt: '||{0})"
                 .format(plpy.quote_literal(str(ex))))
    return None

try:
    rv = plpy.execute("select binfs._statdir({0}, {1}) as stats"
                      .format(plpy.quote_literal(root), len(opt)), 1)
    if rv.nrows() < 1:
        return None
    stats = json.loads(rv[0]['stats'])
except Exception as ex:
    plpy.execute("select core.log('binfs._stats','error','statdir: '||{0})"
                 .format(plpy.quote_literal(str(ex))))
    return None

return {
	"size":			stats['osize'],
	"count":		stats['ocount'],
	"ftimemin":		datetime.utcfromtimestamp(float(stats['oftimemin'])) if stats['oftimemin'] else None,
	"ftimemax":		datetime.utcfromtimestamp(float(stats['oftimemax'])) if stats['oftimemax'] else None,
	"pathmin":		stats['opathmin'],
	"pathmax":		stats['opathmax']
}
$$ language plpythonu;

--
--
--
create or replace function binfs.restat() returns void as $$
begin
perform true from core.lock where name = 'binfs.restat' for update nowait;
update core.lock set lastlock = clock_timestamp() where name = 'binfs.restat';
		insert into binfs.stats select now(), r.name, (binfs._stat(row_to_json(r)::text)).* from (select * from binfs.store) r;
	update core.lock set lastrelease = clock_timestamp() where name = 'binfs.restat';
exception when lock_not_available then
	perform core.log('binfs.restat','warning','lock exceed');
end;	
$$ language plpgsql;

---
--- last state:
--- select distinct on (store) * from binfs.stats order by store, moment desc


