
CREATE OR REPLACE FUNCTION plate_attr_get(
    IN i_plate text,
    IN i_from_ts timestamp without time zone DEFAULT '-infinity'::timestamp without time zone,
    IN i_to_ts timestamp without time zone DEFAULT 'infinity'::timestamp without time zone,
	IN i_distance integer default 0,
    IN i_image character varying DEFAULT 'none'::character varying)
  RETURNS TABLE(
	event_id bigint, 
	event_moment timestamp with time zone, 
	event_post_id uuid, 
	event_post_name text,
	event_channel integer, 
	event_channel_name text, 
	event_media_id bigint, 
	event_plate varchar, 
	event_about text, 
	event_image bytea) AS
$BODY$

declare
 regex varchar;
begin 

if i_from_ts isnull then i_from_ts := '-infinity'::timestamp without time zone; end if;
if i_to_ts isnull   then i_to_ts   := ' infinity'::timestamp without time zone; end if;

regex := i_plate;

if( i_plate ~ E'[^\\.]\\*' or i_plate ~ E'^\*' ) then
	regex := replace(i_plate,'*','.*');
end if;

if ( regex !~ E'^\\^' ) then
	regex := E'^'||regex;
end if;	

if ( regex !~ E'\\$$' ) then
	regex := regex||E'$';
end if;

if i_plate ~ E'[\\[\\]\\^\\$\\\\\\.\\\\+\\*\\?]' then
	return query
	select 
		 platear.id
		,platear.ctime
		,platear.media_id
		,platear.channel
		,channel.description
		,platear.post_id
		,platear.number
		,platear.comment
		,case when i_image = 'none' then null else plate_image_get(platear.post_id,platear.channel,platear.media_id, i_image) end as event_image	
	from
		platear,channel,post 
	where
		post.id = platear.post_id and
		channel.post = platear.post_id and
		channel.channel = platear.channel and
		platear.number ~ regex and
		i_from_ts <= platear.ctime and platear.ctime < i_to_ts; 
elsif i_distance > 0 then
	return query
	select 
		 platear.id
		,platear.ctime
		,platear.media_id
		,platear.channel
		,channel.description
		,platear.post_id
		,platear.number
		,platear.comment
		,case when i_image = 'none' then null else plate_image_get(platear.post_id,platear.channel,platear.media_id, i_image) end as event_image	
	from
		platear,channel,post 
	where
		post.id = platear.post_id and
		channel.post = platear.post_id and
		channel.channel = platear.channel and
		lib.ilevenshtein(platear.number,i_plate) <= i_distance and
		i_from_ts <= platear.ctime and platear.ctime < i_to_ts;
else		
	return query
	select 
		 platear.id
		,platear.ctime
		,platear.media_id
		,platear.channel
		,channel.description
		,platear.post_id
		,platear.number
		,platear.comment
		,case when i_image = 'none' then null else plate_image_get(platear.post_id,platear.channel,platear.media_id, i_image) end as event_image	
	from
		platear,channel,post 
	where
		post.id = platear.post_id and
		channel.post = platear.post_id and
		channel.channel = platear.channel and
		platear.number = i_plate and
		i_from_ts <= platear.ctime and platear.ctime < i_to_ts; 
end if;
return;

end;
$BODY$
  LANGUAGE plpgsql VOLATILE
  COST 100
  ROWS 1000;
