
create or replace function plate_image_get ( ipost uuid, ichannel integer, imid bigint, itype varchar ) returns bytea as $$
	select binfs.load (store,imid) from plate_mediastore where post = ipost and channel = ichannel and image = itype;
$$ language sql;

create or replace function plate_image_set ( ipost uuid, ichannel integer, imid bigint, itype varchar, iover boolean, idata bytea ) returns integer as $$
    select plate_image_reset_meta ( ipost, ichannel, imid, itype, '/digest', json_build_object(
        'size', octet_length(idata),
        'md5', encode(ct.digest(idata,'md5'),'hex'),
        'sha1', encode(ct.digest(idata,'sha1'),'hex'),
        'sha256', encode(ct.digest(idata,'sha256'),'hex')
    ));
	select binfs.save (store,imid,idata,iover) from plate_mediastore where post = ipost and channel = ichannel and image = itype;
$$ language sql;	

create or replace function plate_image_path ( ipost uuid, ichannel integer, imid bigint, itype varchar ) returns text as $$
	select binfs.path (store,imid) from plate_mediastore where post = ipost and channel = ichannel and image = itype;
$$ language sql;


create or replace function plate_image_get_meta ( ipost uuid, ichannel integer, imid bigint, itype varchar ) returns json as $$
	select json(binfs.meta_get(store,imid)) from plate_mediastore where post = ipost and channel = ichannel and image = itype;
$$ language sql;

create or replace function plate_image_set_meta ( ipost uuid, ichannel integer, imid bigint, itype varchar, iover boolean, idata json ) returns integer as $$
	select binfs.meta_set (store,imid,idata::text,iover) from plate_mediastore where post = ipost and channel = ichannel and image = itype;
$$ language sql;	

create or replace function plate_image_reset_meta ( ipost uuid, ichannel integer, imid bigint, itype varchar, ipath text, idata json ) returns json as $$
declare
	jdata json;
begin
	select into jdata plate_image_get_meta(ipost,ichannel,imid,itype);
	if jdata isnull then jdata := json('{}'); end if;
	select into jdata lib.json_path_set(jdata,ipath,idata);
	perform plate_image_set_meta (ipost,ichannel,imid,itype,true,jdata);
	return jdata;
end;
$$ language plpgsql;	



